/**
 * This file is distributed as part of the code accompanying the paper:
 *
 * "Robust Region Detection via Consensus Segmentation of Deformable Shapes"
 * E. Rodola, S. Rota Bulo, and D. Cremers
 * Computer Graphics Forum, 33(5), 2014
 *
 * If you use this code, please cite the paper above.
 *
 * to compile:
 * mex -v -O hash_keys.cpp -I../boost_1_54_0
 *
 * Emanuele Rodolà (c) 2014
 */
#include "mex.h"
#include <cmath>
#include <vector>
#include <iostream>
#include <boost/unordered_map.hpp>

typedef boost::unordered_map< std::vector<int>, double > keymap_t;

void mexFunction(int nlhs, mxArray *plhs[], int nrhs, const mxArray*prhs[])
{
	if (nrhs != 2 || nlhs != 2)
		mexErrMsgTxt("Usage: [unique_keys, weights] = hash_keys(keys, tri_weights).");

	const double* const keys = mxGetPr(prhs[0]);
	const int nt = int( mxGetN(prhs[0]) );
	const int len = int( mxGetM(prhs[0]) );
	const double* const tri_weights = mxGetPr(prhs[1]);

	if (nt < len)
		mexErrMsgTxt("It seems like you have less triangles than key length. Maybe you must transpose?");
		
	if ( std::min(mxGetM(prhs[1]),mxGetN(prhs[1])) != 1 )
		mexErrMsgTxt("The input triangle weights should be either a row or a column vector.");
		
	if ( std::max(mxGetM(prhs[1]),mxGetN(prhs[1])) != nt )
		mexErrMsgTxt("The input weights should be one per triangle.");
	
	keymap_t map;
	std::vector<int> key(len);
	
	for (int i=0; i<nt; ++i)
	{
		for (int j=0; j<len; ++j)
			key[j] = keys[i*len+j];

		if (map.find(key) == map.end())
			map[key] = tri_weights[i];
		else
			map[key] += tri_weights[i];
	}
	
	plhs[0] = mxCreateDoubleMatrix(map.size(), len, mxREAL);
	double* unique_keys = mxGetPr(plhs[0]);
	
	plhs[1] = mxCreateDoubleMatrix(map.size(), 1, mxREAL);
	double* weights = mxGetPr(plhs[1]);
	
	int i=0;
	for (keymap_t::const_iterator it=map.begin(); it!=map.end(); ++it, ++i)
	{
		const std::vector<int>& curkey = it->first;
		for (int j=0; j<len; ++j)
			unique_keys[i+j*map.size()] = curkey[j];
		weights[i] = it->second;
	}
}
